#include "huc.h"


/* VRAM locations */

#define VRAM_BLOCK1       0x3000
#define VRAM_BLOCK2       0x3010
#define VRAM_BLOCK3       0x3020
#define VRAM_BLOCK4       0x3030
#define VRAM_LINE1        0x3040
#define VRAM_LINE2        0x3050
#define VRAM_LINE3        0x3060
#define VRAM_LINE4        0x3070
#define VRAM_CORNER1      0x3080
#define VRAM_CORNER2      0x3090
#define VRAM_CORNER3      0x30A0
#define VRAM_CORNER4      0x30B0
#define VRAM_PLAYCHAR     0x30C0
#define VRAM_FLASHCHAR    0x30D0

#define VRAM_BLOCK1_PATT       0x0300
#define VRAM_BLOCK2_PATT       0x0301
#define VRAM_BLOCK3_PATT       0x0302
#define VRAM_BLOCK4_PATT       0x0303
#define VRAM_LINE1_PATT        0x0304
#define VRAM_LINE2_PATT        0x0305
#define VRAM_LINE3_PATT        0x0306
#define VRAM_LINE4_PATT        0x0307
#define VRAM_CORNER1_PATT      0x0308
#define VRAM_CORNER2_PATT      0x0309
#define VRAM_CORNER3_PATT      0x030A
#define VRAM_CORNER4_PATT      0x030B
#define VRAM_PLAYCHAR_PATT     0x030C
#define VRAM_FLASHCHAR_PATT    0x030D

#define VRAM_CHEVVERT     0x3800
#define VRAM_CHEVHORZ     0x3840
#define VRAM_CHEVSPLAT    0x3880
#define VRAM_PLAYVERT     0x38C0
#define VRAM_PLAYHORZ     0x3900
#define VRAM_PLAYSPLAT1   0x3940
#define VRAM_PLAYSPLAT2   0x3980
#define VRAM_PLAYSPLAT3   0x39C0
#define VRAM_SHOTVERT     0x3A00
#define VRAM_SHOTHORZ     0x3A40


#include "attack.h"



/* miscellaneous constants */

#define DIR_UP		0
#define DIR_LEFT	1
#define DIR_DOWN	2
#define DIR_RIGHT	3

/*************************************************************/
/* Note: (x,y) positions are tracked in units of 1/4 pixel   */
/*       so that motions of less than 1 pixel per VSYNC      */
/*       are possible.                                       */
/*                                                           */
/*       Minimum speed resolution is 1 pixel per 4 VSYNCs    */
/*                                                           */
/* Edge locations are also tracked in these 1/4 pixel units  */ 
/*************************************************************/

#define LEFT_EDGE	16
#define RIGHT_EDGE	880
#define TOP_EDGE	80
#define BOTTOM_EDGE	752

#define CAR_COLLIDE_GAP		32
#define	SHOT_COLLIDE_GAP	24

#define PLAYER_PARTSPEEDMAX	2
#define PLAYER_PARTSPEEDMIN	1
#define PLAYER_STOP_TIME	12


/* constants for bad guy 'alive' values */
/* explosion is displayed if value is between DEAD and ALIVE */

#define BAD_DEAD	0
#define BAD_EXPLODE	4
#define BAD_ALIVE	10


/* sprite numbers in use */
/* chevrons are 1-10 */
/* other bad guys between 10 and PLAYSHOT */

#define SPRITE_PLAYER	0
#define SPRITE_PLAYSHOT	32
#define NUM_CHEVRONS	10


/* graphic palettes */

#define BGPAL           0
#define SPRPAL          0


/* include graphics */

/*
background      = 0 = blue
blocks          = 1 = yellow
chevrons        = 2 = red
player          = 3 = green
shot            = 4 = light white
space bad guy   = 5 = light green    *** not yet in the game
*/



/* GLOBALS */

/* attributes for player */

int player_alive;	/* flag for death */
int player_stopped;	/* time that player stopped dead at end of aisle */
int player_part_speed;	/* to slow down the acceleration */
int player_speed;	/* delta position per vsync */
int player_direction;  /* see DIR_xxx defines */
int player_x_old;	/* old position */
int player_y_old;
int player_x;		/* new position */
int player_y;


/* attributes for shot */

int shot_alive;	/* flag for shot in play */
int shot_direction;	/* see DIR_xxx defines */
int shot_speed;	/* delta position per vsync */
int shot_x;		/* position */
int shot_y;


/* attributes for bad guy */

int bad_value;		/* score amount per kill */
int bad_alive[16];	/* array of bad guys - alive or not */
int bad_speed[16];	/* delta per vsync */
int bad_direction[16];	/* see DIR_xxx defines */
int bad_x[16];		/* position */
int bad_y[16];


/* scores */

int move;		/* joystick sample */
int score;		/* Player's score */
int hiscore;		/* Highest Player's score */
int lives;		/* Number of player cars remaining */
char badguys;		/* current number of badguys on board, alive */


/* level-based data */

int level;		/* number of screens completed */
int player_max_speed;	/* level-based player's maximum speed */
int badguy_speed;	/* level-based badguy's speed */


/* miscellaneous */

int vram_flash[16];	/* special for flashing effect */


/* These are the locations of turn aisles in 1/4 pixel units */
/* cols are vertical columns (noted by x-axis locations) */
/* rows are horizontal rows (noted by y-axis locations) */

#define MAX_COLS  9
#define MAX_ROWS  7

const int cols[] = { 16, 112, 208, 304, 400, 496, 592, 688, 784, 880 };
const int rows[] = { 80, 176, 272, 368, 464, 560, 656, 752 };

const int value[] = { 10, 20, 30, 40, 50, 60, 70, 80, 90, 100 };
const int bonus[] = { 200, 300, 400, 500, 750, 1000, 1250, 1500, 2000, 3000 };
const int bad_spd[] = { 7, 8, 8, 9, 10, 11, 11, 12, 13, 13 };
const int ply_spd[] = { 8, 8, 7, 7,  7,  7,  6,  6,  6,  5 };

const int numbers[] = { zero, one, two, three, four, five, six, seven, eight, nine };

const char zero[] = {
1, 1, 1, 1,
1, 0, 0, 1, 
1, 0, 0, 1, 
1, 0, 0, 1, 
1, 1, 1, 1 };

const char one[] = {
0, 0, 1, 0,
0, 1, 1, 0,
0, 0, 1, 0,
0, 0, 1, 0,
0, 1, 1, 1
};

const char two[] = {
1, 1, 1, 0,
0, 0, 0, 1,
0, 1, 1, 0,
1, 0, 0, 0,
1, 1, 1, 1
};

const char three[] = {
1, 1, 1, 0,
0, 0, 0, 1,
0, 1, 1, 0,
0, 0, 0, 1,
1, 1, 1, 0
};

const char four[] = {
1, 0, 1, 0,
1, 0, 1, 0,
1, 1, 1, 1,
0, 0, 1, 0,
0, 0, 1, 0
};

const char five[] = {
1, 1, 1, 1,
1, 0, 0, 0,
1, 1, 1, 0,
0, 0, 0, 1,
1, 1, 1, 0
};

const char six[] = {
0, 1, 1, 1,
1, 0, 0, 0,
1, 1, 1, 1,
1, 0, 0, 1,
1, 1, 1, 1
};

const char seven[] = {
1, 1, 1, 1,
0, 0, 0, 1,
0, 0, 1, 0,
0, 1, 0, 0,
1, 0, 0, 0
};

const char eight[] = {
0, 1, 1, 0,
1, 0, 0, 1,
0, 1, 1, 0,
1, 0, 0, 1,
0, 1, 1, 0
};

const char nine[] = {
0, 1, 1, 0,
1, 0, 0, 1,
0, 1, 1, 1,
0, 0, 0, 1,
1, 1, 1, 0
};

/* opposite direction, when using direction as a subscript */

const char opp_direction[] = { DIR_DOWN, DIR_RIGHT, DIR_UP, DIR_LEFT };


/* --------- START ATTACK ---------- */


/* execute the game from setup board until endgame */

bonus_message(points)
int points;
{
static int i, indx, indy, x, y, rem;
static char *ptr;

   cls();
   put_string("BONUS   ", 20, 15);
   put_string("POINTS   ", 20, 16);

   rem = points % 10;
   indx = 15;
   indy = 13;

   while (points != 0)
   {
      ptr = numbers[rem];
      for (y = 0; y < 5; y++)
      {
         for (x = 0; x < 4; x++)
         {
            if (*ptr == 1)
            {
               put_raw(VRAM_FLASHCHAR_PATT, indx+x, indy+y);
            }
            ptr++;
         }
      }
      points = points / 10;
      rem = points % 10;
      indx -= 5;
   }

   i = 0;
   while (i < 60)
   {
      for (x = 0; x < 8; x++)
      {
         rem = rand() & 0xff;
         rem = (rem << 8) | rem;

         vram_flash[x] = rem;
         vram_flash[x+8] = rem;
      }
      load_vram(VRAM_FLASHCHAR, vram_flash, 16);

      i++;
      vsync(3);
   }
   cls();
}


pause_message()
{
   put_string("           ", 10, 13);
   put_string("   PAUSE   ", 10, 14);
   put_string("           ", 10, 15);
}


unpause_message()
{
   put_string("           ", 10, 14);
   init_board();
}


gameover_message()
{
   dispscore();
   vsync(60);
   put_string("             ", 9, 13);
   put_string("  GAME OVER  ", 9, 14);
   put_string("             ", 9, 15);

   if (score > hiscore)
   {
      put_string("  NEW HISCORE:       ", 5, 16);
      put_string("                     ", 5, 17);
      hiscore = score;
      put_number(hiscore, 5, 19, 16);
   }
   vsync(120);

   /* erase words that we wrote */
   put_string("             ", 9, 14);
   put_string("                     ", 5, 16);
}


start_message()
{
   /* draw empty board */
   init_board();

   /* put caption and wait */
   put_string("              ", 8, 13);
   put_string("  START GAME  ", 8, 14);
   put_string("              ", 8, 15);
   vsync(120);

   /* erase words that we wrote */
   put_string("              ", 8, 14);

   /* redraw empty board */
   init_board();
}


init_board()
{
static int x,y;

   /* place block-barriers inside grid */

   for (y = 4; y < 25; y+=3)
   {
      for (x = 2; x < 29; x+=3)
      {
         put_raw((VRAM_BLOCK1_PATT), x, y);
         put_raw((VRAM_BLOCK2_PATT), x+1, y);
         put_raw((VRAM_BLOCK3_PATT), x, y+1);
         put_raw((VRAM_BLOCK4_PATT), x+1, y+1);
      }
   }

   /* draw barrier around play field */

   for (y = 3; y < 25; y++)
   {
      put_raw((VRAM_LINE2_PATT), 0, y);
      put_raw((VRAM_LINE3_PATT), 29, y);
   }
   for (x = 1; x < 29; x++)
   {
      put_raw((VRAM_LINE1_PATT), x, 2);
      put_raw((VRAM_LINE4_PATT), x, 25);
   }

   put_raw((VRAM_CORNER1_PATT), 0, 2);
   put_raw((VRAM_CORNER2_PATT), 29, 2);
   put_raw((VRAM_CORNER3_PATT), 0, 25);
   put_raw((VRAM_CORNER4_PATT), 29, 25);
}


init_pieces(num_of_badguys)
int num_of_badguys;
{
static int i, st;

   player_alive = 1;
   player_stopped = 0;
   player_part_speed = PLAYER_PARTSPEEDMIN;
   player_speed = 1;
   player_direction = DIR_LEFT;
   player_x = RIGHT_EDGE-8;
   player_y = BOTTOM_EDGE;

   shot_speed = 16;
   shot_alive = 0;

   bad_value        = value[level];
   badguy_speed     = bad_spd[level];
   player_max_speed = ply_spd[level];

   sprite_player();
   sprite_shot();

   for (i = 0; i < 16; i++)
   {
      bad_alive[i] = 0;
      bad_speed[i] = 0;
      bad_direction[i] = 0;
      bad_x[i] = 0;
      bad_y[i] = 0;
   }

   /* if two bad guys, point opposite ways at top of screen */

   if (num_of_badguys == 2)
   {
      bad_alive[0]     = BAD_ALIVE;
      bad_speed[0]     = badguy_speed;
      bad_y[0]         = TOP_EDGE;
      bad_direction[0] = DIR_LEFT;
      bad_x[0]         = LEFT_EDGE + (96 * 4);

      bad_alive[1]     = BAD_ALIVE;
      bad_speed[1]     = badguy_speed;
      bad_y[1]         = TOP_EDGE;
      bad_direction[1] = DIR_RIGHT;
      bad_x[1]         = LEFT_EDGE + (96 * 5);
   }
   else
   {
      st = (NUM_CHEVRONS - num_of_badguys) >> 1;

      for (i = 0; i < num_of_badguys; i++)
      {
         bad_alive[i]     = BAD_ALIVE;
         bad_speed[i]     = badguy_speed;
         bad_y[i]         = TOP_EDGE;
         bad_direction[i] = DIR_DOWN;
         bad_x[i]         = LEFT_EDGE + (96 * (i+st));

         sprite_chev(i);
      }
   }
   satb_update();
}


/* display score */

dispscore()
{
static int i;

   put_number(score, 5, 2, 1);
   put_string("HI", 21, 1);
   put_number(hiscore, 5, 24, 1);

   put_string("         ", 9, 1);
   for (i = 0; i < lives; i++)
   {
      put_raw((VRAM_PLAYCHAR_PATT), 9+i, 1);
   }
}


sprite_hideall()
{
static int i;

   for (i = 0; i < 64; i++)
   {
      spr_set(i);
      spr_hide();
   }
   satb_update();
}


sprite_player()
{
   spr_set(SPRITE_PLAYER);
   spr_x(player_x>>2);		/* divide by 4 */
   spr_y(player_y>>2);		/* divide by 4 */

   spr_pal(0);
   spr_pri(1);

   switch (player_direction)
   {
   case DIR_UP:
      spr_pattern(VRAM_PLAYVERT);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
      break;

   case DIR_LEFT:
      spr_pattern(VRAM_PLAYHORZ);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
      break;

   case DIR_DOWN:
      spr_pattern(VRAM_PLAYVERT);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|FLIP_Y);
      break;

   case DIR_RIGHT:
      spr_pattern(VRAM_PLAYHORZ);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|FLIP_X);
      break;
   }
}


sprite_shot()
{
   spr_set(SPRITE_PLAYSHOT);

   if (shot_alive == 0) {
      spr_hide();
   }
   else
   {
      spr_x(shot_x>>2);	/* divide by 4 */
      spr_y(shot_y>>2);	/* divide by 4 */

      spr_pal(0);
      spr_pri(1);

      switch (shot_direction)
      {
      case DIR_UP:
         spr_pattern(VRAM_SHOTVERT);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
         break;

      case DIR_LEFT:
         spr_pattern(VRAM_SHOTHORZ);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
         break;

      case DIR_DOWN:
         spr_pattern(VRAM_SHOTVERT);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|FLIP_Y);
         break;

      case DIR_RIGHT:
         spr_pattern(VRAM_SHOTHORZ);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|FLIP_X);
         break;
      }
   }
}


sprite_chev(ind)
int ind;
{
   spr_set(ind+1);

   if (bad_alive[ind] == BAD_DEAD) {
      spr_hide();
   }
   else
   {
      spr_x(bad_x[ind]>>2);	/* divide by 4 */
      spr_y(bad_y[ind]>>2);	/* divide by 4 */

      spr_pal(0);
      spr_pri(1);

      if (bad_alive[ind] < BAD_ALIVE)	/* dying - so count down to DEAD */
      {
         spr_pattern(VRAM_CHEVSPLAT);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
      }

      else switch (bad_direction[ind])	/* else alive and moving */
      {
      case DIR_UP:
         spr_pattern(VRAM_CHEVVERT);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
         break;

      case DIR_LEFT:
         spr_pattern(VRAM_CHEVHORZ);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
         break;

      case DIR_DOWN:
         spr_pattern(VRAM_CHEVVERT);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|FLIP_Y);
         break;

      case DIR_RIGHT:
         spr_pattern(VRAM_CHEVHORZ);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|FLIP_X);
         break;
      }
   }
}


shot_move()
{
   /* if no shot in play, then allow the player to */
   /* shoot, else no reason to continue processing */

   if (shot_alive == 0)
   {
      if ((move & JOY_A) == 0)
         return(0);

      shot_alive = 1;
      shot_x = player_x;
      shot_y = player_y;
      shot_direction = player_direction;
   }

   /* shot in play -> so move it */
   /* if it hit a wall, it's out of play */

   switch (shot_direction)
   {
   case DIR_UP:
      shot_y -= shot_speed;

      if (shot_y < TOP_EDGE-8) {
         shot_alive = 0;
      }
      break;

   case DIR_DOWN:
      shot_y += shot_speed;

      if (shot_y > BOTTOM_EDGE+8) {
         shot_alive = 0;
      }
      break;

   case DIR_LEFT:
      shot_x -= shot_speed;

      if (shot_x < LEFT_EDGE-8) {
         shot_alive = 0;
      }
      break;

   case DIR_RIGHT:
      shot_x += shot_speed;

      if (shot_x > RIGHT_EDGE+8) {
         shot_alive = 0;
      }
      break;
   }
}


chev_check_row(dir, y_pos)
int dir, y_pos;
{
static int test, index;

   test = 0;

   for (index = 0; index < MAX_ROWS - 1; index++)
   {
      if (abs(rows[index] - y_pos) < badguy_speed)
      {
         if ((dir == DIR_UP) && (y_pos <= rows[index]))
            test = rows[index];

         if ((dir == DIR_DOWN) && (y_pos >= rows[index]))
            test = rows[index];
      }
   }
   return(test);
}


chev_check_column(dir, x_pos)
int dir, x_pos;
{
static int test, index;

   test = 0;

   for (index = 0; index < MAX_COLS - 1; index++)
   {
      if (abs(cols[index] - x_pos) < badguy_speed)
      {
         if ((dir == DIR_LEFT) && (x_pos <= cols[index]))
            test = cols[index];

         if ((dir == DIR_RIGHT) && (x_pos >= cols[index]))
            test = cols[index];
      }
   }
   return(test);
}


chev_up(ind)
int ind;
{
static int rowat;

   bad_y[ind] -= bad_speed[ind];

   rowat = 0;

   if ((rand() & 3) == 3)
   {
      rowat = chev_check_row(DIR_UP, bad_y[ind]);
   }

   if (bad_y[ind] < TOP_EDGE)
   {
      bad_y[ind] = TOP_EDGE;
      rowat = TOP_EDGE;
   }

   if (rowat != 0)
   {
      if (rand() & 1)
      {
         if (bad_x[ind] == LEFT_EDGE)
         {
            bad_direction[ind] = DIR_RIGHT;
            bad_y[ind] = rowat;
         }
         else
         {
            bad_direction[ind] = DIR_LEFT;
            bad_y[ind] = rowat;
         }
      }
      else
      {
         if (bad_x[ind] == RIGHT_EDGE)
         {
            bad_direction[ind] = DIR_LEFT;
            bad_y[ind] = rowat;
         }
         else
         {
            bad_direction[ind] = DIR_RIGHT;
            bad_y[ind] = rowat;
         }
      }
   }
}


chev_down(ind)
int ind;
{
static int rowat;

   bad_y[ind] += bad_speed[ind];

   rowat = 0;

   if ((rand() & 3) == 3)
   {
      rowat = chev_check_row(DIR_DOWN, bad_y[ind]);
   }

   if (bad_y[ind] > BOTTOM_EDGE)
   {
      bad_y[ind] = BOTTOM_EDGE;
      rowat = BOTTOM_EDGE;
   }

   if (rowat != 0)
   {
      if (rand() & 1)
      {
         if (bad_x[ind] == LEFT_EDGE)
         {
            bad_direction[ind] = DIR_RIGHT;
            bad_y[ind] = rowat;
         }
         else
         {
            bad_direction[ind] = DIR_LEFT;
            bad_y[ind] = rowat;
         }
      }
      else
      {
         if (bad_x[ind] == RIGHT_EDGE)
         {
            bad_direction[ind] = DIR_LEFT;
            bad_y[ind] = rowat;
         }
         else
         {
            bad_direction[ind] = DIR_RIGHT;
            bad_y[ind] = rowat;
         }
      }
   }
}


chev_left(ind)
int ind;
{
static int columat;

   bad_x[ind] -= bad_speed[ind];

   columat = 0;

   if ((rand() & 3) == 3)
   {
      columat = chev_check_column(DIR_LEFT, bad_x[ind]);
   }

   if (bad_x[ind] < LEFT_EDGE)
   {
      bad_x[ind] = LEFT_EDGE;
      columat = LEFT_EDGE;
   }

   if (columat != 0)
   {
      if (rand() & 1)
      {
         if (bad_y[ind] == TOP_EDGE)
         {
            bad_direction[ind] = DIR_DOWN;
            bad_x[ind] = columat;
         }
         else
         {
            bad_direction[ind] = DIR_UP;
            bad_x[ind] = columat;
         }
      }
      else
      {
         if (bad_y[ind] == BOTTOM_EDGE)
         {
            bad_direction[ind] = DIR_UP;
            bad_x[ind] = columat;
         }
         else
         {
            bad_direction[ind] = DIR_DOWN;
            bad_x[ind] = columat;
         }
      }
   }
}


chev_right(ind)
int ind;
{
static int columat;

   bad_x[ind] += bad_speed[ind];

   columat = 0;

   if ((rand() & 3) == 3)
   {
      columat = chev_check_column(DIR_RIGHT, bad_x[ind]);
   }

   if (bad_x[ind] > RIGHT_EDGE)
   {
      bad_x[ind] = RIGHT_EDGE;
      columat = RIGHT_EDGE;
   }

   if (columat != 0)
   {
      if (rand() & 1)
      {
         if (bad_y[ind] == TOP_EDGE)
         {
            bad_direction[ind] = DIR_DOWN;
            bad_x[ind] = columat;
         }
         else
         {
            bad_direction[ind] = DIR_UP;
            bad_x[ind] = columat;
         }
      }
      else
      {
         if (bad_y[ind] == BOTTOM_EDGE)
         {
            bad_direction[ind] = DIR_UP;
            bad_x[ind] = columat;
         }
         else
         {
            bad_direction[ind] = DIR_DOWN;
            bad_x[ind] = columat;
         }
      }
   }
}


chev_move(ind)
int ind;
{
static int i,x,y;
static char x_match;

   /* actual movement */

   switch (bad_direction[ind])
   {
   case DIR_UP:
      chev_up(ind);
      break;

   case DIR_DOWN:
      chev_down(ind);
      break;

   case DIR_LEFT:
      chev_left(ind);
      break;

   case DIR_RIGHT:
      chev_right(ind);
      break;
   }

   x = bad_x[ind];
   y = bad_y[ind];
   x_match = 0;


   /* Check if collided with player */

   if ( (abs(x-player_x) < CAR_COLLIDE_GAP) &&
        (abs(y-player_y) < CAR_COLLIDE_GAP) )
   {
      player_alive = 0;
   }


   /* Check if shot impacted it */

   x_match = 0;

   if ( (shot_alive == 1) &&
        (abs(x-shot_x) < SHOT_COLLIDE_GAP) &&
        (abs(y-shot_y) < SHOT_COLLIDE_GAP) )
   {
      bad_alive[ind] = BAD_EXPLODE;
      shot_alive = 0;
      score += bad_value;
   }


   /* Check if collided with another bad guy */

   for (i = ind; i < NUM_CHEVRONS; i++)
   {
      if (bad_alive[i] == BAD_DEAD)
         continue;

      if ( (abs(x-bad_x[i]) < CAR_COLLIDE_GAP) &&
           (abs(y-bad_y[i]) < CAR_COLLIDE_GAP) )
      {
         bad_direction[ind] = opp_direction[bad_direction[ind]];
         bad_direction[i]   = opp_direction[bad_direction[i]];
      }
   }
}


player_inc_speed()
{
   player_part_speed++;
   if (player_part_speed > PLAYER_PARTSPEEDMAX)
   {
      player_part_speed = PLAYER_PARTSPEEDMIN;
      if (player_speed < player_max_speed) {
         player_speed++;
      }
   }
}


player_dec_speed(dir)
int dir;
{
   player_part_speed--;
   if (player_part_speed < PLAYER_PARTSPEEDMIN)
   {
      player_part_speed = PLAYER_PARTSPEEDMAX;
      if (player_speed > 1) {
         player_speed--;
      }
      else {
         player_direction = opp_direction[dir];
         player_stopped = 0;
      }
   }
}


player_lr_turn(dir, turndir)
int dir, turndir;
{
static int test, index;

   test = 0;

   for (index = 0; index < MAX_ROWS+1; index++)
   {
      if (abs(rows[index] - player_y) < player_speed)
      {
         if ((dir == DIR_UP) && (player_y <= rows[index]))
            test = 1;

         if ((dir == DIR_DOWN) && (player_y >= rows[index]))
            test = 1;
      }

      if (test == 1)
      {
         if ((turndir & JOY_LEFT) && (player_x != LEFT_EDGE))
         {
            player_y = rows[index];
            player_direction = DIR_LEFT;
            player_stopped = 0;
            break;
         }
         else if ((turndir & JOY_RGHT) && (player_x != RIGHT_EDGE))
         {
            player_y = rows[index];
            player_direction = DIR_RIGHT;
            player_stopped = 0;
            break;
         }
      }
   }
}


player_ud_turn(dir, turndir)
int dir, turndir;
{
static int test, index;

   test = 0;

   for (index = 0; index < MAX_COLS+1; index++)
   {
      if (abs(cols[index] - player_x) < player_speed)
      {
         if ((dir == DIR_LEFT) && (player_x <= cols[index]))
            test = 1;

         if ((dir == DIR_RIGHT) && (player_x >= cols[index]))
            test = 1;
      }

      if (test == 1)
      {
         if ((turndir & JOY_UP) && (player_y != TOP_EDGE))
         {
            player_x = cols[index];
            player_direction = DIR_UP;
            player_stopped = 0;
            break;
         }
         else if ((turndir & JOY_DOWN) && (player_y != BOTTOM_EDGE))
         {
            player_x = cols[index];
            player_direction = DIR_DOWN;
            player_stopped = 0;
            break;
         }
      }
   }
}


player_up()
{
   if (!player_stopped)
      player_y -= player_speed;

   if (move & (JOY_LEFT | JOY_RGHT))
   {
      player_lr_turn(DIR_UP, move);
   }
   else if (move & JOY_UP)
   {
      player_inc_speed();
   }
   else if (move & JOY_DOWN)
   {
      player_dec_speed(DIR_UP);
   }

   if (player_y <= TOP_EDGE)
   {
      player_y = TOP_EDGE;

      if (player_direction == DIR_UP)
      {
         player_stopped++;
         if (player_stopped > PLAYER_STOP_TIME)
         {
            player_direction = DIR_RIGHT;
            player_stopped = 0;
         }
      }
   }
}


player_down()
{
   if (!player_stopped)
      player_y += player_speed;

   if (move & (JOY_LEFT | JOY_RGHT))
   {
      player_lr_turn(DIR_DOWN, move);
   }
   else if (move & JOY_DOWN)
   {
      player_inc_speed();
   }
   else if (move & JOY_UP)
   {
      player_dec_speed(DIR_DOWN);
   }

   if (player_y >= BOTTOM_EDGE)
   {
      player_y = BOTTOM_EDGE;

      if (player_direction == DIR_DOWN)
      {
         player_stopped++;
         if (player_stopped > PLAYER_STOP_TIME)
         {
            player_direction = DIR_LEFT;
            player_stopped = 0;
         }
      }
   }
}


player_left()
{
   if (!player_stopped)
      player_x -= player_speed;

   if (move & (JOY_UP | JOY_DOWN))
   {
      player_ud_turn(DIR_LEFT, move);
   }
   else if (move & JOY_LEFT)
   {
      player_inc_speed();
   }
   else if (move & JOY_RGHT)
   {
      player_dec_speed(DIR_LEFT);
   }

   if (player_x <= LEFT_EDGE)
   {
      player_x = LEFT_EDGE;

      if (player_direction == DIR_LEFT)
      {
         player_stopped++;
         if (player_stopped > PLAYER_STOP_TIME)
         {
            player_direction = DIR_UP;
            player_stopped = 0;
         }
      }
   }
}


player_right()
{
static int index;

   if (!player_stopped)
      player_x += player_speed;

   if (move & (JOY_UP | JOY_DOWN))
   {
      player_ud_turn(DIR_RIGHT, move);
   }
   else if (move & JOY_RGHT)
   {
      player_inc_speed();
   }
   else if (move & JOY_LEFT)
   {
      player_dec_speed(DIR_RIGHT);
   }

   if (player_x >= RIGHT_EDGE)
   {
      player_x = RIGHT_EDGE;

      if (player_direction == DIR_RIGHT)
      {
         player_stopped++;
         if (player_stopped > PLAYER_STOP_TIME)
         {
            player_direction = DIR_DOWN;
            player_stopped = 0;
         }
      }
   }
}


gameloop()
{
static int i;

   while (1)
   {
/*      set_color_rgb(256, 0, 0, 0);*/	/* check performance on real machine */

      satb_update();

/*      set_color_rgb(256, 0, 0, 6);*/	/* check performance on real machine */

      vsync(2);
      dispscore();

/*      set_color_rgb(256, 7, 7, 7);*/	/* check performance on real machine */


      /* if dead, stop looping */

      if (player_alive == 0) {
          break;
      }

      /* based on joystick delta, move pointer */

      move = joy(0);
      if (move & JOY_STRT)
      {
         pause_message();
         vsync(0);
         while ((joytrg(0) & JOY_STRT) == 0);	/* wait for RUN key press */
         unpause_message();
         while (joy(0) & JOY_STRT);		/* wait for RUN key release */
      }

      player_x_old = player_x;
      player_y_old = player_y;


      /* different logic depending on which direction you're pointing */

      switch(player_direction)
      {
      case DIR_UP:
         player_up();
         break;

      case DIR_DOWN:
         player_down();
         break;

      case DIR_LEFT:
         player_left();
         break;

      case DIR_RIGHT:
         player_right();
         break;
      }


      /* move player's sprite */

      sprite_player();

      /* and sprite of player's shot */

      shot_move();
      sprite_shot();


      /* now do badguys, and count them while you're at it */

      badguys = 0;

      for (i = 0; i < NUM_CHEVRONS; i++)
      {
         if (bad_alive[i] == BAD_ALIVE)
         {
            badguys++;
            chev_move(i);
            sprite_chev(i);
         }
         else if (bad_alive[i] > 0)
         {
            badguys++;
            bad_alive[i]--;
            sprite_chev(i);
         }
      }

      if (badguys == 0) {
         break;
      }
   }
}


playgame()
{
static char i;
static char animate;

   cls();

   init_satb();

   set_sprpal(SPRPAL, colors, 2);
   set_bgpal(BGPAL, colors, 2);


   /* setup initial values */

   score = 0;
   lives = 3;
   level = 0;
   badguys = NUM_CHEVRONS;

/* set up playing surface */

   init_pieces(badguys);
   dispscore();
   start_message();	/* does an init_board also */


/* MAIN LOOP */

   while (lives > 0)
   {
      gameloop();

      if (badguys == 0)
      {
         sprite_hideall();
         bonus_message(bonus[level]);
         score += bonus[level];
         level++;

         badguys = NUM_CHEVRONS;
         init_pieces(badguys);
         init_board();
         dispscore();
         vsync(120);
      }
      else
      {
         spr_set(SPRITE_PLAYER);
         spr_pattern(VRAM_PLAYSPLAT1);
         spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
         satb_update();
         vsync(5);

         spr_pattern(VRAM_PLAYSPLAT2);
         satb_update();
         vsync(5);

         spr_pattern(VRAM_PLAYSPLAT3);
         satb_update();
         vsync(60);

         lives--;
         if (lives > 0)
         {
            player_alive = 1;
            sprite_hideall();
            init_pieces(badguys);
            init_board();
         }
         else
         {
            gameover_message();
         }
      }
   }
   sprite_hideall();
}


titlescreen()
{
static char select;

   select = 0;

/* This will display the titlescreen soon */

   cls();


   put_string("ATTACK", 12, 10);
   put_string("By Dave Shadoff", 7, 12);

   put_string("Press RUN to Start", 6, 18);

   while (1)
   {
       vsync(0);

       if (joytrg(0) == JOY_STRT)
       {
           return(0);
       }
   }
}



main()
{

/* load some basic stuff */

/*   set_xres(240, XRES_SOFT); */
   set_xres(240);
   hiscore = 0;

   set_sprpal(SPRPAL, colors, 2);
   set_bgpal(BGPAL, colors, 2);

/* load character data for squares */

   load_vram(VRAM_BLOCK1, block1, 16);
   load_vram(VRAM_BLOCK2, block2, 16);
   load_vram(VRAM_BLOCK3, block3, 16);
   load_vram(VRAM_BLOCK4, block4, 16);
   load_vram(VRAM_LINE1, line1, 16);
   load_vram(VRAM_LINE2, line2, 16);
   load_vram(VRAM_LINE3, line3, 16);
   load_vram(VRAM_LINE4, line4, 16);
   load_vram(VRAM_CORNER1, corner1, 16);
   load_vram(VRAM_CORNER2, corner2, 16);
   load_vram(VRAM_CORNER3, corner3, 16);
   load_vram(VRAM_CORNER4, corner4, 16);
   load_vram(VRAM_PLAYCHAR, player_char, 16);

/* load data for selection pointer */

   load_sprites(VRAM_CHEVVERT, chevvert, 1);
   load_sprites(VRAM_CHEVHORZ, chevhorz, 1);
   load_sprites(VRAM_CHEVSPLAT, chevsplat, 1);

   load_sprites(VRAM_PLAYVERT, playvert, 1);
   load_sprites(VRAM_PLAYHORZ, playhorz, 1);
   load_sprites(VRAM_PLAYSPLAT1, playsplat1, 1);
   load_sprites(VRAM_PLAYSPLAT2, playsplat2, 1);
   load_sprites(VRAM_PLAYSPLAT3, playsplat3, 1);

   while (1)
   {
       titlescreen();
       playgame();
   }
}

